var express = require('express')
var xml = require('js2xmlparser')
var router = express.Router()

var libs = process.cwd() + '/libs/'

var log = require(libs + 'log')(module)
var config = require(libs + 'config')

var conn = require(libs + 'db/monet')

router.get('/', function (req, res) {
    res.json({
        msg: 'SimCAQ API is running'
    })
})

/**
 * Complete range of the enrollments dataset
 *
 * Returns a tuple of start and ending years of the complete enrollments dataset.
 */
router.get('/year_range', function(req, res) {
    var yearSql = "SELECT MIN(t.ano_censo) AS start_year, MAX(t.ano_censo) AS end_year FROM turmas AS t";
    conn.query(yearSql, true).then(function(result) {
      if (req.query.format === 'csv') {
        res.csv(result.data);
      } else if (req.query.format === 'xml') {
        res.send(xml("result", JSON.stringify({year_range: result.data})))
      }
      else {
        res.json({
            result: result.data
        });
      }
    });
})

router.get('/data', function(req, res) {
    log.debug(req.query)
    log.debug(req.query.met)
    log.debug(req.query.dim)
    conn.query(
        'SELECT * FROM turmas'
    ).then(function(result) {
      if (req.query.format === 'csv') {
        res.csv(result.data);
      } else if (req.query.format === 'xml') {
        res.send(xml("result", JSON.stringify({data: result.data})))
      }
      else {
        res.json({
            result: result.data
        });
      }
    })
})

router.get('/enrollments', function(req, res) {
    var params = req.query;
    var id = 0;
    var location_id = 0;
    var adm_dependency_id = 0;
    var census_year = 0;
    var enrollmentSql = "";

    if (params.id)
    {
        id = parseInt(params.id, 10);
    }

    if (params.location_id)
    {
        location_id = parseInt(params.location_id, 10);
    }

    if (params.adm_dependency_id)
    {
        adm_dependency_id = parseInt(params.adm_dependency_id, 10);
    }

    if (params.census_year)
    {
        census_year = parseInt(params.census_year, 10);
    }

    /**
     * FIXME: parameter substitution in the queries is not safe (vulnerable to
     * SQL injection). Substitution from MonetDB module is not working for some
     * reason.
     */
    switch(params.aggregate)
    {
    case "city":
        if (id) {
            enrollmentSql = "SELECT nome AS name, total FROM mat_municipio(" + id + "," + census_year + "," + adm_dependency_id + "," + location_id + ")";
        } else {
            enrollmentSql = "SELECT nome AS name, total FROM mat_municipios(" + census_year + "," + adm_dependency_id + "," + location_id + ")";
        }
        break;
    case "state":
        if (id) {
            enrollmentSql = "SELECT nome AS name, total FROM mat_estado(" + id + "," + census_year + "," + adm_dependency_id + "," + location_id + ")";
        } else {
            enrollmentSql = "SELECT nome AS name, total FROM mat_estados(" + census_year + "," + adm_dependency_id + "," + location_id + ")";
        }
        break;
    case "region":
        if (id) {
            enrollmentSql = "SELECT nome AS name, total FROM mat_regiao(" + id + "," + census_year + "," + adm_dependency_id + "," + location_id + ")";
        } else {
            enrollmentSql = "SELECT nome AS name, total FROM mat_regioes(" + census_year + "," + adm_dependency_id + "," + location_id + ")";
        }
        break;
    default:
        enrollmentSql = "SELECT nome AS name, total FROM mat_brasil(" + census_year + "," + adm_dependency_id + "," + location_id + ")";
    }

    log.debug(params);
    log.debug("Executing query: " + enrollmentSql);

    conn.query(enrollmentSql, true).then(function(result) {
        log.debug(result);
        if (req.query.format === 'csv') {
          res.csv(result.data);
        } else if (req.query.format === 'xml') {
          res.send(xml("result", JSON.stringify({enrollments: result.data})))
        }
        else {
          res.json({
              result: result.data
          });
        }
        log.debug("All resources were released");
    });
})

module.exports = router
